<?php

namespace Drupal\vuejs\Form;

use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Vue.js settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * The library.discovery service.
   *
   * @var \Drupal\Core\Asset\LibraryDiscoveryInterface
   */
  protected $libraryDiscovery;

  /**
   * Constructs a form object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config.factory service.
   * @param \Drupal\Core\Asset\LibraryDiscoveryInterface $library_discovery
   *   The library.discovery service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    LibraryDiscoveryInterface $library_discovery
  ) {
    $this->libraryDiscovery = $library_discovery;
    parent::__construct($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('library.discovery')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'vuejs_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['vuejs.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $registered_libraries = $this->libraryDiscovery->getLibrariesByExtension('vuejs');

    $libraries = $this->config('vuejs.settings')->get('libraries');

    foreach ($libraries as $library_name => $library) {
      $form[$library_name] = [
        '#type' => 'fieldset',
        '#title' => str_replace('_', '-', $library_name),
        '#tree' => TRUE,
        '#description' => $this->t("<strong>Source options:</strong><dl>
        <dt>Disabled:</dt><dd>Do not provide this library at all</dd>
        <dt>Local library (manual download or via drush):</dt><dd>Download the library manually or by <code>'drush vuejs:download &lt;LIBRARY_NAME&gt;'</code> and place it in libraries/vuejs/&lt;LIBRARY_NAME>/&lt;VERSION&gt;/&lt;LIBRARY_NAME&gt;(.min).js</dd>
        <dt>Local via Composer (Asset Packagist: NPM):</dt><dd>Download libraries by <code>'composer require npm-asset/vue:&lt;VERSION&gt; npm-asset/vue-resource:&lt;VERSION&gt; npm-asset/vue-router:&lt;VERSION&gt;'</code> to libraries/&lt;LIBRARY_NAME>/dist/&lt;LIBRARY_NAME&gt;(.min).js. See <a href='@composerAssetsInfoUrl'>this drupal.org Documentation</a> for details.</dd>
        <dt>Local via Composer (Asset Packagist: Bower):</dt><dd>Download libraries by <code>'composer require bower-asset/vue:&lt;VERSION&gt; bower-asset/vue-resource:&lt;VERSION&gt; bower-asset/vue-router:&lt;VERSION&gt;'</code> to libraries/&lt;LIBRARY_NAME>/dist/&lt;LIBRARY_NAME&gt;(.min).js. See <a href='@composerAssetsInfoUrl'>this drupal.org Documentation</a> for details.</dd>
        <dt>CDNJS:</dt><dd>Loads the library from remote CDN <a href='https://cdnjs.com/'>https://cdnjs.com/</a></dd>
        <dt>jsDelivr:</dt><dd>Loads the library from remote CDM <a href='https://www.jsdelivr.com/'>https://www.jsdelivr.com/</a></dd>
        </dl>
        <em>Note: vue 3.x has a different file structure and is currently NOT supported. See <a href='https://www.drupal.org/project/vuejs/issues/3196859'>this</a> issue.</em>
        ", [
          '@composerAssetsInfoUrl' => 'https://www.drupal.org/docs/develop/using-composer/using-composer-to-install-drupal-and-manage-dependencies#third-party-libraries'
        ]),
      ];

      $form[$library_name]['installation'] = [
        '#type' => 'select',
        '#title' => $this->t('Installation'),
        '#options' => [
          // Do not provide this library:
          'disabled' => $this->t('Disabled'),
          // Local library downloaded manually or by drush:
          // drush vuejs:download vue / vue-router / vue-resource
          'local' => $this->t('Local library (manual download or via drush)'),
          // Local library downloaded via composer using bower-asset/vue
          // See https://www.drupal.org/docs/8/modules/lazy-load/how-to-use-composer-to-install-lazy-load-module-and-its-dependency
          // Note: We have to use bower-asset insted of npm-asset as npm-asset
          // downloads unwanted further dependencies while bower-asset only
          // provides the wanted UDM packages:
          'local_composer_npm' => $this->t('Local library via Composer (Asset Packagist: NPM)'),
          'local_composer_bower' => $this->t('Local library via Composer (Asset Packagist: Bower)'),
          // Load from CDN: CDNJS
          'cdnjs' => $this->t('CDNJS'),
          // Load from CDN: jsDelivr
          'jsdelivr' => $this->t('jsDelivr'),
        ],
        '#default_value' => $library['installation'],
      ];

      $form[$library_name]['development'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Development version'),
        '#default_value' => $library['development'],
        '#states' => array(
          'invisible' => array(
            'select[name="' . $library_name . '[installation]"]' => array(
              'value' => 'disabled',
            ),
          ),
        ),
      ];

      $form[$library_name]['version'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Version'),
        '#size' => 9,
        '#required' => TRUE,
        '#default_value' => $library['version'],
        '#states' => array(
          'invisible' => array(
            'select[name="' . $library_name . '[installation]"]' => array(
              array('value' => 'disabled'),
              array('value' => 'local_composer_npm'),
              array('value' => 'local_composer_bower'),
            ),
          ),
        ),
      ];

      $path = $registered_libraries[$library_name]['js'][0]['data'];
      $options = [
        'absolute' => TRUE,
        'attributes' => ['target' => '_blank'],
      ];
      if (strpos($path, 'libraries') === 0) {
        $url = Url::fromUserInput('/' . $path, $options);
      }
      else {
        $url = Url::fromUri($path, $options);
      }

      $form[$library_name]['url'] = [
        '#type' => 'item',
        '#title' => $this->t('URL'),
        '#markup' => (new Link($url->toString(), $url))->toString(),
        '#states' => array(
          'invisible' => array(
            'select[name="' . $library_name . '[installation]"]' => array('value' => 'disabled'),
          ),
        ),
      ];

    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    $libraries = $this->config('vuejs.settings')->get('libraries');
    foreach ($libraries as $library_name => $library) {

      $value = $form_state->getValue($library_name);

      if (!preg_match('/^\d+\.\d+\.\d+$/', $value['version'])) {
        $form_state->setErrorByName($library_name . '][version', $this->t('Version format is not correct.'));
      }

      $form_state->setValue($library_name, $value);
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('vuejs.settings')
      ->set('libraries.vue', $form_state->getValue('vue'))
      ->set('libraries.vue_router', $form_state->getValue('vue_router'))
      ->set('libraries.vue_resource', $form_state->getValue('vue_resource'))
      ->save();

    $this->libraryDiscovery->clearCachedDefinitions();
    parent::submitForm($form, $form_state);
  }

}
